// Player.cpp

#include "Player.h"

Player::Player(int nr) : m_nr(nr)
{
	int i;
	string texFilename;
	
	for(i=0; i<NUM_P_TEXS; i++)
	{
		texFilename = "images/player";

		if(i == 0)
			texFilename.push_back(nr + '0');
		else
		{
			texFilename.push_back(nr + '0');
			texFilename.append("hurt");
			texFilename.push_back(i + '0');
		}

		texFilename.append(".bmp");
		m_textures[i] = LoadTexture(texFilename.c_str(), true);
	}

	m_hurtState = 0;

	switch(nr)
	{
		case 1:
			m_keyLeft = SDLK_LEFT;
			m_keyRight = SDLK_RIGHT;
			m_keyUp = SDLK_UP;
			m_keyDown = SDLK_DOWN;
			m_keyShoot = SDLK_RCTRL;

			m_x = 10;
			m_y = 10;
			break;
		case 2:
			m_keyLeft = SDLK_a;
			m_keyRight = SDLK_d;
			m_keyUp = SDLK_w;
			m_keyDown = SDLK_s;
			m_keyShoot = SDLK_LCTRL;

			m_x = SCR_W - PLAYER_WIDTH - 10;
			m_y = SCR_H - PLAYER_HEIGHT - 10;
			break;
	}

	m_lShot = SDL_GetTicks();
}

Player::~Player()
{
	int i;
	m_bullets.clear();
	for(i=0; i<NUM_P_TEXS; i++)
		FreeTexture(&m_textures[i]);
}

void Player::Input(Uint8 *keys)
{
	if(keys[m_keyLeft])
	{
		m_x -= SPEED;

		if(m_x < 0)
			m_x = 0;
	}

	if(keys[m_keyRight])
	{
		m_x += SPEED;

		if(m_x + PLAYER_WIDTH > SCR_W)
			m_x = SCR_W - PLAYER_WIDTH;
	}

	if(keys[m_keyUp])
	{
		m_y -= SPEED;

		if(m_y < 0)
			m_y = 0;
	}

	if(keys[m_keyDown])
	{
		m_y += SPEED;

		if(m_y + PLAYER_HEIGHT > SCR_H)
			m_y = SCR_H - PLAYER_HEIGHT;
	}

	if(keys[m_keyShoot])
		Shoot();

	switch(m_nr)
	{
		case 1:
			if(m_x + PLAYER_WIDTH > SCR_W / 2)
				m_x = SCR_W / 2 - PLAYER_WIDTH;
			break;
		case 2:
			if(m_x < SCR_W / 2)
				m_x = SCR_W / 2;
			break;
	}
}

void Player::Update()
{
	list<Bullet>::iterator it;
	static int movAdd;
	
	switch(m_nr)
	{
		case 1:
			movAdd = BULLET_SPEED;
			break;
		case 2:
			movAdd = -BULLET_SPEED;
			break;
	}
	
	for(it = m_bullets.begin(); it != m_bullets.end(); it++)
	{
		it->x += movAdd;
		
		if(it->x > SCR_W || it->x + BULLET_WIDTH < 0)
			it = m_bullets.erase(it);
	}
}

void Player::Draw()
{
	list<Bullet>::iterator it;
	
	DrawTexture(m_textures[m_hurtState], m_x, m_y, PLAYER_WIDTH, PLAYER_HEIGHT, true);
	
	for(it = m_bullets.begin(); it != m_bullets.end(); it++)
		DrawTexture(bulletTex, it->x, it->y, BULLET_WIDTH, BULLET_HEIGHT, true);
}

void Player::Shoot()
{
	if(SDL_GetTicks() - m_lShot > RELOAD_TIME)
	{
		Bullet newBullet = { m_x, m_y + (PLAYER_HEIGHT - BULLET_HEIGHT) / 2 };
		
		if(m_nr == 1)
			newBullet.x += PLAYER_WIDTH;
		
		m_bullets.push_back(newBullet);
		m_lShot = SDL_GetTicks();
	}
}

bool Player::BulletCollWith(Player *player)
{
	list<Bullet>::iterator it, itOther;
	list<Bullet> *otherBullets = player->GetBullets();
	SDL_Rect bulletRect = { 0, 0, BULLET_WIDTH, BULLET_HEIGHT };
	SDL_Rect otherBulletRect = { 0, 0, BULLET_WIDTH, BULLET_HEIGHT };
	SDL_Rect playerRect = { player->GetX(), player->GetY(), PLAYER_WIDTH, PLAYER_HEIGHT };
	
	for(it = m_bullets.begin(); it != m_bullets.end(); it++)
	{
		bulletRect.x = it->x;
		bulletRect.y = it->y;
		
		if(Coll(&playerRect, &bulletRect))
		{
			it = m_bullets.erase(it);
			return true;
		}
		
		for(itOther = otherBullets->begin(); itOther != otherBullets->end(); itOther++)
		{
			otherBulletRect.x = itOther->x;
			otherBulletRect.y = itOther->y;
			
			if(Coll(&otherBulletRect, &bulletRect))
			{
				itOther = otherBullets->erase(itOther);
				it = m_bullets.erase(it);
			}			
		}
	}
	
	return false;
}

void Player::Hit()
{
	m_hurtState++;
	
	if(m_hurtState == NUM_P_TEXS - 1)
		done = true;
}
