// Utility.cpp

#include "Utility.h"

void Error(const char *msg)
{
	printf("Error: %s\n", msg);
	fflush(stdout);
	exit(1);
}

GLuint LoadTexture(const char *filename, bool useAlpha)
{
	SDL_Surface *temp, *image;
	GLuint texture;
	int format;

	temp = IMG_Load(filename);
	if(!temp)
	{
		printf("Filename: %s\n", filename);
		Error("Can't load image!");
	}

	if(useAlpha)
	{
		Uint32 magenta = SDL_MapRGB(temp->format, 255, 0, 255);
		SDL_SetColorKey(temp, SDL_SRCCOLORKEY, magenta);
		format = 4;
	}
	else
		format = 3;

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
	image = SDL_CreateRGBSurface(	SDL_SWSURFACE, temp->w, temp->h, SCR_BPP,
									0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
#else
	image = SDL_CreateRGBSurface(	SDL_SWSURFACE, temp->w, temp->h, SCR_BPP,
									0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
#endif

	SDL_BlitSurface(temp, NULL, image, NULL);
	SDL_FreeSurface(temp);

	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, IMAGE_FILTER);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, IMAGE_FILTER);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, image->pitch / image->format->BytesPerPixel);
	glTexImage2D(GL_TEXTURE_2D, 0, format, image->w, image->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);

	SDL_FreeSurface(image);

	return texture;
}

void DrawTexture(GLuint texture, int x, int y, int w, int h, bool useBlend)
{
	glBindTexture(GL_TEXTURE_2D, texture);

	if(useBlend)
	{
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4ub(255, 255, 255, 160);
	}

	glBegin(GL_QUADS);
		glTexCoord2i(0, 0); glVertex2i(x, y);
		glTexCoord2i(1, 0); glVertex2i(x + w, y);
		glTexCoord2i(1, 1); glVertex2i(x + w, y + h);
		glTexCoord2i(0, 1); glVertex2i(x, y + h);
	glEnd();
}

void FreeTexture(GLuint *texture)
{
	glDeleteTextures(1, texture);
}

bool Coll(SDL_Rect *rect1, SDL_Rect *rect2)
{
	return 	(rect1->x + rect1->w > rect2->x) &&
			(rect1->x < rect2->x + rect2->w) &&
			(rect1->y + rect1->h > rect2->y) &&
			(rect1->y < rect2->y + rect2->h);
}
